__author__ = 'arnout'

from plot import plot, plotasync, line, area, spline, pie
from server import run_server

from IPython.core import getipython
from IPython.core.display import display, HTML

import options
import data
import jsonencoder



__author__ = 'Arnout Aertgeerts'

import json


class ChartsJSONEncoder(json.JSONEncoder):
    def default(self, obj):
        try:
            import numpy
            if isinstance(obj, numpy.ndarray):
                if obj.ndim == 1:
                    return obj.tolist()
                else:
                    return [self.default(obj[i]) for i in range(obj.shape[0])]
            elif isinstance(obj, numpy.generic):
                return obj.item()
        except ImportError:
            pass

        return json.JSONEncoder.default(self, obj)



__author__ = 'Arnout Aertgeerts'

import SimpleHTTPServer
import SocketServer
import os
import json

class ChartRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):

    def __init__(self, request, client_address, server):
        SimpleHTTPServer.SimpleHTTPRequestHandler.__init__(self, request, client_address, server)
        return

    def do_POST(self):
        content_len = int(self.headers.getheader('content-length', 0))
        body = json.loads(self.rfile.read(content_len))

        with open(body['name'], 'w') as chart_file:
            chart_file.write(body['svg'])

        SimpleHTTPServer.SimpleHTTPRequestHandler.do_GET(self)

    def log_message(self, format, *args):
        pass


class ChartServer(SocketServer.TCPServer):

    def __init__(self, server_address, handler_class=ChartRequestHandler):
        SocketServer.TCPServer.__init__(self, server_address, handler_class)
        return


def run_server():
    import threading

    address = ('localhost', 0)  # let the kernel give us a port
    server = ChartServer(address, ChartRequestHandler)
    ip, port = server.server_address  # find out what port we were given

    print 'Server running in the folder {0} at {1}:{2}'.format(os.path.abspath(os.getcwd()), ip, port)

    t = threading.Thread(target=server.serve_forever)
    t.setDaemon(True)  # don't hang on exit
    t.start()

    return 'http://{0}:{1}/'.format(ip, port)


address = run_server()


def url(path):
    return address + path



__author__ = 'arnoutaertgeerts'


from core import to_series, to_json_files, show_plot, set_display


class Chart():
    def __init__(self, inline, html, path, show):
        self.inline = inline
        self.html = html
        self.path = path
        self.show_property = show

    def plot(self, *args, **kwargs):
        if len(args) == 2:
            self._plot_single(*args, **kwargs)
        else:
            self._plot_multi(*args, **kwargs)

    def _plot_multi(self, series, display):
        series = to_series(series)
        series = set_display(series, display)

        to_json_files(series, self.path)

    def _plot_single(self, data, name, display):
        series = to_series(dict(data=data, name=name))
        series = set_display(series, display)

        to_json_files(series, self.path, )

    def show(self):
        return show_plot(self.inline, self.html, self.show_property, async=self.path)


__author__ = 'Arnout Aertgeerts'

from core import MyTemplate, to_json_files, to_series, clean_dir, set_display, show_plot
from jsonencoder import ChartsJSONEncoder
from chart import Chart
from server import address

import os
import json

package_directory = os.path.dirname(os.path.abspath(__file__))

TABDEPS = """
    <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.4/css/bootstrap.min.css"/>
    <script src="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.4/js/bootstrap.min.js"></script>
"""

def line(*args, **kwargs):
    return plot(*args, type='line', **kwargs)


def area(*args, **kwargs):
    return plot(*args, type='area', **kwargs)


def spline(*args, **kwargs):
    return plot(*args, type='spline', **kwargs)


def pie(*args, **kwargs):
    return plot(*args, type='pie', **kwargs)


def stock(*args, **kwargs):
    return plot(*args, stock=True, **kwargs)


def plot(
        series, options=dict(), type='line', name=False,
        height=400, save=False, stock=False, show='tab', display=True):
    """
    Make a highchart plot with all data embedded in the HTML
    :param type: Type of the chart (will overwrite options['chart']['type'] if specified).
    :param series: The necessary data, can be a list of dictionaries or a dataframe
    :param options: Options for the chart
    :param height: Chart height
    :param save: Specify a filename to save the HTML file if wanted.
    :param stock: Set to False to use Highcharts instead of highstock
    :param show: Determines how the chart is shown. Can be one of the following options:
        - 'tab': Show the chart in a new tab of the default browser
        - 'window': Show the chart in a new window of the default browser
        - 'inline': Show the chart inline (only works in IPython notebook)
    :param display: A list containing the keys of the variables you want to show initially in the plot
    :return:
    """
    try:
        if options['chart']:
            options['chart'].update(dict(type=type))
    except KeyError:
        options['chart'] = dict(type=type)

    # Convert to a legitimate series object
    series = to_series(series, name)

    # Set the display option
    series = set_display(series, display)

    # Get the save extension
    if save:
        extension = os.path.splitext(save)[1]
    else:
        extension = False

    saveSVG = False
    saveHTML = False

    if extension == '.svg':
        saveSVG = save
    if show != 'inline':
        saveHTML = 'index.html'
    if extension == '.html':
        saveHTML = save

    with open(os.path.join(package_directory, "index.html"), "r") as html:
        html = MyTemplate(html.read()).substitute(
            path=package_directory,
            series=json.dumps(series, cls=ChartsJSONEncoder),
            options=json.dumps(options),
            highstock=json.dumps(stock),
            height=str(height) + "px",
            url=json.dumps(address),
            save=json.dumps(saveSVG)
        )

    if saveHTML:
        with open(saveHTML, "w") as text_file:
            text_file.write(html + TABDEPS)

    return show_plot(html, saveHTML, show)


def plotasync(
        series=None, options=dict(), type='line',
        height=400, save="temp", stock=False, show='tab', display=False, purge=False, live=False):
    """
    :param type: Type of the chart. Can be line, area, spline, pie, bar, ...
    :param display: Set to true to display all, False to display none or an array of names for a specific selection
    :param purge: Set to true to clean the directory
    :param live: Set to true to keep the chart in sync with data in the directory. Currently only works for show='tab'
    :param series: The series object which contains the data. If this is not specified, the plot will look for json
                   files in the save directory.
    :param options: The chart display options
    :param height: Height of the chart
    :param save: Name of the directory to store the data
    :param stock: Set to true to use highstock
    :param show: Determines how the chart is shown. Can be one of the following options:
        - 'tab': Show the chart in a new tab of the default browser
        - 'window': Show the chart in a new window of the default browser
        - 'inline': Show the chart inline (only works in IPython notebook)
    :return: A chart object
    """

    try:
        if not options['chart']:
            options['chart'] = dict(type=type)
    except KeyError:
        options['chart'] = dict(type=type)

    # Clean the directory
    if purge:
        clean_dir(save)

    if series:
        # Convert to a legitimate series object
        series = to_series(series)
        series = set_display(series, display)

        # Convert to json files
        to_json_files(series, save)

    if show == 'inline':
        live = False

    with open(os.path.join(package_directory, "index-async.html"), "r") as index:
        read = index.read()

        html = MyTemplate(read).substitute(
            path=json.dumps('/' + save),
            options=json.dumps(options),
            highstock=json.dumps(stock),
            height=str(height) + "px",
            live=json.dumps(live),
            url=json.dumps(address),
            save=json.dumps(False)
        )

        inline = MyTemplate(read).substitute(
            path=json.dumps(save),
            options=json.dumps(options),
            highstock=json.dumps(stock),
            height=str(height) + "px",
            live=json.dumps(live),
            url=json.dumps(address),
            save=json.dumps(False)
        )

    html_path = os.path.join(save, 'index.html')
    with open(html_path, "w") as html_file:
        html_file.write(html + TABDEPS)

    return Chart(inline, html_path, save, show)



__author__ = 'arnoutaertgeerts'

from string import Template
from jsonencoder import ChartsJSONEncoder
from server import url

import os
import json
import shutil
import webbrowser


class MyTemplate(Template):
    delimiter = '$#'
    idpattern = r'[a-z][_a-z0-9]*'


def show_plot(html, saveHTML, show, async=False):
    if show == 'inline':
        from IPython.display import HTML
        return HTML(html)

    elif show == 'tab':
        print 'Opening new tab...'
        if async:
            address = url(async)
            webbrowser.open_new_tab(address)
        else:
            webbrowser.open_new_tab('file://' + os.path.realpath(saveHTML))

    elif show == 'window':
        print 'Trying to open a window. If this fails we will open a tab...'
        if async:
            address = url(async)
            webbrowser.open_new(address)
        else:
            webbrowser.open_new('file://' + os.path.realpath(saveHTML))


def clean_dir(path):
    if os.path.exists(path):
        shutil.rmtree(path)
    os.makedirs(path)


def to_json_files(series, path):
    try:
        with open(os.path.join(path, 'keys.json'), "r") as keys_file:
            keys = json.loads(keys_file.read())
    except IOError:
        keys = []

    for s in series:
        if s["name"] not in map(lambda x: x["name"], keys):
            keys.append(dict(name=s["name"], display=s["display"], value=s["name"], text=s["name"]))
            with open(os.path.join(path, s["name"] + ".json"), "w") as json_file:
                json_file.write(json.dumps(s, cls=ChartsJSONEncoder))
        else:
            print "Careful: '%s' was already present as key and is not added to the chart!" % s["name"]

    with open(os.path.join(path, 'keys.json'), "w") as keys_file:
        keys_file.write(json.dumps(keys))


def set_display(series, display):
    if display is True:
        for s in series:
            s['display'] = True

    elif display is False:
        for s in series:
            s['display'] = False

    else:
        for s in series:
            if s['name'] in display:
                s['display'] = True
            else:
                s['display'] = False

    return series


def df_to_series(df):
    """Prepare data from dataframe for plotting with python-highcharts.
    all columns in df are entries in the returned series.

    The returned series is in the format suitable for python-highcharts: list of dicts with:
    data:list of [index, value]-lists.
    name:name of variable.
    """

    import pandas as pd
    try:
        df.index = df.index.tz_localize(None)
        index = [int(x/1e6) for x in df.index.asi8]
    except Exception as e:
        index = df.index

    df = df.where((pd.notnull(df)), None)
    series = []
    for col in df:
        data = []
        for i, x in enumerate(index):
            try:
                data.append([int(x), df[col].iloc[i]])
            except ValueError:
                data.append([x, df[col].iloc[i]])
        my_dict = {'data': data,'name': col}
        series.append(my_dict)
    return series


def list_to_series(array):
    return dict(
        data=array
    )


def to_series(series, name=False):
    # Dictionary?
    if isinstance(series, dict):
        return [series]

    # List of dictionaries?:
    try:
        if isinstance(series[0], dict):
            return series
    except KeyError:
        pass

    # List?
    try:
        if isinstance(series, list):
            return [dict(data=series, name=name)]
    except KeyError:
        pass

    # Numpy array?
    try:
        import numpy as np
        if isinstance(series, np.ndarray):
            return [dict(data=series, name=name)]
    except ImportError:
        pass

    # DataFrame?:
    import pandas as pd
    if isinstance(series, pd.DataFrame):
        return df_to_series(series)

    raise ValueError('Your data is not in the right format!')



__author__ = 'Arnout Aertgeerts'


def default():
    options = {
        'chart': {
            'zoomType': 'y'
        },

        'tooltip': {
            'decimalValues': 2
        },
        'title': {
            'text': 'Some test data'
        },
        'legend': {
            'enabled': 'true'
        },
        'rangeSelector': {
            'buttons': [
            {
                'type': 'day',
                'count': '1',
                'text': '1d'

            }, {
                'type': 'day',
                'count': '7',
                'text': '1w'

            }, {
                'type': 'month',
                'count': '1',
                'text': '1m'
            }, {
                'type': 'month',
                'count': '3',
                'text': '3m'
            }, {
                'type': 'month',
                'count': '6',
                'text': '6m'
            }, {
                'type': 'year',
                'count': '1',
                'text': '1y'
            }, {
                'type': 'all',
                'text': 'All'
            }]
        }
    }
    return options


def area():
    return {
        'chart': {'type': 'area'},
        'title': {'text': 'Area chart'}
    }


__author__ = 'arnout'

from setuptools import setup, find_packages

setup(
    name='charts',
    version='0.4.0',
    description='Use the highcharts js library in Python',
    url='https://github.com/arnoutaertgeerts/python-highcharts',
    author='Arnout Aertgeerts',
    author_email='arnoutaertgeerts@gmail.com',
    license='MIT',
    keywords='highcharts highstock plotting',
    packages=find_packages(),
    install_requires=[],
    package_data={
        'charts': ['*.html']
    }
)